﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Xaml;
using System.Windows.Markup;
using System.Windows.Media.Imaging;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.Wpf;
using alib.String;
using Microsoft.Win32;
using Microsoft.WindowsAPICodePack.Dialogs;

namespace xigt.app
{
	public partial class app_menu : Menu
	{
		public app_menu()
		{
			InitializeComponent();
		}

		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e);

			if (e.Property == NameScope.NameScopeProperty)
				((INameScope)e.NewValue).RegisterName("S", Settings.Instance);
		}

		void Font_Click(Object sender, RoutedEventArgs e)
		{
			App.main.FontFamily = new FontFamily((String)((MenuItem)sender).Header);
		}

		void Menu_New(Object sender, RoutedEventArgs e) { App.main.NewCorpus(); }

		void Menu_Open(Object sender, RoutedEventArgs e) { App.main.OpenFile(); }

		void Menu_Load(Object sender, RoutedEventArgs e)
		{
			var mi = (FrameworkElement)e.OriginalSource;
			App.main.LoadFile((String)mi.DataContext);
		}

		void Menu_Save(Object sender, RoutedEventArgs e) { App.main.SaveFile(); }

		void Menu_Close(Object sender, RoutedEventArgs e) { App.main.CloseFile(); }

		void Menu_SaveAs(Object sender, RoutedEventArgs e) { App.main.SaveFileAs(); }

		void Menu_SaveAllFiles(Object sender, RoutedEventArgs e) { App.main.SaveAllFiles(); }

		void Menu_ResetSettings(Object sender, RoutedEventArgs e) { Settings.Instance.Reset(); }

		void Menu_CloseAll(Object sender, RoutedEventArgs e) { App.main.CloseAllFiles(); }

		void Menu_Exit(Object sender, RoutedEventArgs e) { App.main.Close(); }

		void Menu_Exit_nosave(Object sender, RoutedEventArgs e)
		{
			App.main.f_force_discard = true;
			App.main.Close();
		}

#if false
		void Menu_SaveAllFilesTo(Object sender, RoutedEventArgs e)
		{
			var dlg = new CommonOpenFileDialog
			{
				Title = "Set directory for output files",
				InitialDirectory = Path.GetDirectoryName(App.settings.LastDirectory),
				DefaultFileName = Path.GetFileName(App.settings.LastDirectory),
				IsFolderPicker = true,
			};

			if (dlg.ShowDialog(w) == CommonFileDialogResult.Ok)
			{
				var fn = dlg.FileName;
				App.settings.LastDirectory = fn;

				if (w.retarget_all_files_to(fn))
				{
					int c_saved = w.save_all_files();

					var msg = String.Format("{0} XAML-IGT files saved to '{1}'", c_saved, fn);
					MessageBox.Show(w, msg, "Saved XAML-IGT files", MessageBoxButton.OK, MessageBoxImage.Information);
				}
			}
		}

		void Menu_OpenXamlIgtDir(Object sender, RoutedEventArgs _)
		{
			var dir = App.settings.LastDirectory;
			if (!Directory.Exists(dir))
				dir = Environment.CurrentDirectory;

			var dlg = new CommonOpenFileDialog
			{
				Title = "Select xaml-igt directory",
				InitialDirectory = Path.GetDirectoryName(dir),
				IsFolderPicker = true,
				EnsureFileExists = true,
			};

			var e = Directory.EnumerateFiles(dir, "*.xml", SearchOption.AllDirectories).GetEnumerator();
			while (e.MoveNext())
				if (!(dlg.DefaultFileName = Path.GetFileName(Path.GetDirectoryName(e.Current))).StartsWith("."))
					goto ok;
			dlg.DefaultFileName = null;
		ok:

			if (dlg.ShowDialog(w) == CommonFileDialogResult.Ok)
			{
				dir = dlg.FileName;

				w.cmd_OpenXamlIgtDir(dir);

				App.settings.LastDirectory = dir;
			}
		}

		void Menu_OpenXamlIgtFile(object _, RoutedEventArgs __)
		{
			OpenFileDialog dlg = new OpenFileDialog
			{
				Title = "Open xaml-igt file...",
				DefaultExt = ".xml",
				Filter = "Xaml Interlinear Glossed Text (*.xml)|*.xml|All files (*.*)|*.*",
				InitialDirectory = App.settings.LastDirectory,
				AddExtension = true,
				CheckFileExists = true,
			};

			if (dlg.ShowDialog(w).Value)
			{
				var fn = dlg.FileName;
				//App.settings.LastFilename = fn;
				App.settings.LastDirectory = Path.GetDirectoryName(fn);
				w.cmd_OpenXamlIgtFile(fn);
			}
		}
#endif
	};
}
