﻿using System;
using System.Diagnostics;
using System.IO;
using System.Collections.Generic;
using System.Reflection;
using System.Linq;
using System.Xml;
using System.Xaml;
using System.ComponentModel;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;

using alib;
using alib.Debugging;

namespace xigt.app
{
	public partial class App : Application
	{
		public static readonly Random rnd;

		public static readonly XamlSchemaContext xsch;

		public static main main { [DebuggerStepThrough] get { return Application.Current.MainWindow as main; } }

		static App()
		{
			rnd = new Random();

			if (Path.GetFileName(Application.ResourceAssembly.Location) == "XDesProc.exe")
				Environment.CurrentDirectory = @"D:\Programming\xigt-edit";

			xsch = new XamlSchemaContext(new XamlSchemaContextSettings
			{
				SupportMarkupExtensionsWithDuplicateArity = true,
			});
		}

		public static bool FindConfigFile(ref String filename)
		{
			String fn1, fn2;
			var dir1 = Environment.CurrentDirectory;
			if (File.Exists(fn1 = Path.Combine(dir1, filename)))
			{
				filename = fn1;
				return true;
			}

			var dir2 = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
			if (File.Exists(fn2 = Path.Combine(dir2, filename)))
			{
				filename = fn2;
				return true;
			}

			filename = fn1;
			return false;
		}

		public static void save_to_xaml(Object o, String filename)
		{
			using (var sw = XmlWriter.Create(filename, new XmlWriterSettings
			{
				Indent = true,
				NewLineOnAttributes = true,
				NamespaceHandling = NamespaceHandling.OmitDuplicates,
				OmitXmlDeclaration = true,
			}))
			using (var xr = new XamlObjectReader(o, xsch))
			using (var xw = new XamlXmlWriter(sw, xsch))
			{
				XamlServices.Transform(xr, xw);
			}
		}

		public static Object _load(String fn)
		{
			Object ret;
			using (var sr = new StreamReader(fn))
			using (var xr = new XamlXmlReader(sr, xsch))
			using (var xw = new XamlObjectWriter(xsch))
			{
				XamlServices.Transform(xr, xw);

				ret = xw.Result;
			}
			return ret;
		}

		protected override void OnExit(ExitEventArgs e)
		{
			Settings.Instance.Save();

			base.OnExit(e);
		}
	};
}
