﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Input;
using System.Diagnostics;
using System.Linq;
using System.Windows;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.Wpf;

using Microsoft.WindowsAPICodePack.Dialogs;

namespace xigt.app
{
	public partial class main
	{
		///////////////////////////////////////////////////////
		/// 
		void mw_click_new(Object sender, RoutedEventArgs e) { NewCorpus(); }
		public void NewCorpus()
		{
			var xc = xigt_corpus.New();
			corpora.Add(xc);
			corpora.SelectedItem = xc;

			w_corpora.FocusSelectedItem();
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_open(Object sender, RoutedEventArgs e) { OpenFile(); }
		public void OpenFile()
		{
			var dlg = new CommonOpenFileDialog
			{
				Title = String.Format("Open Xigt-XML file..."),
				InitialDirectory = Settings.Instance.LastDirectory,
				DefaultExtension = ".xml",
				EnsureFileExists = true,
				Multiselect = true,
				Filters =
				{
					new CommonFileDialogFilter("Xigt-XML",".xml"),
					new CommonFileDialogFilter("All files",".*")
				}
			};

			if (dlg.ShowDialog(this) == CommonFileDialogResult.Ok)
				LoadFiles(dlg.FileNames);
		}

		///////////////////////////////////////////////////////
		/// 
		public bool LoadFile(String file)
		{
			if (file == null || !LoadFiles(new[] { file }))
				return false;

			w_corpora.FocusSelectedItem();
			return true;
		}
		public bool LoadFiles(IEnumerable<String> files)
		{
			if (files == null)
				return false;

			bool f_any = false;
			xigt_corpus xc = null;

			foreach (var fn in files)
			{
				switch (corpora.AddXigtFile(fn, out xc))
				{
					case AddCorpusResult.AlreadyLoaded:
						MessageBox.Show(String.Format("File:\r\r'{0}'\r\ris already loaded.", fn), "File not loaded", MessageBoxButton.OK, MessageBoxImage.Information);
						break;

					case AddCorpusResult.FileNotFound:
						MessageBox.Show(String.Format("File:\r\r'{0}'\r\rwas not found.", fn), "File not loaded", MessageBoxButton.OK, MessageBoxImage.Information);
						break;

					case AddCorpusResult.Ok:
						corpora.SelectedItem = xc;
						Settings.Instance.SetRecentFile(fn);
						f_any = true;
						break;
				}
			}

			return f_any;
		}

		///////////////////////////////////////////////////////
		/// 
		public bool SaveAllFiles()
		{
			foreach (var xc in corpora)
				if (!SaveFile(xc))
					return false;
			return true;
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_save(Object sender, RoutedEventArgs e) { SaveFile(); }
		public bool SaveFile(xigt_corpus xc = null)
		{
			if ((xc ?? (xc = corpora.SelectedItem)) == null)
				return false;

			return xc.FullPath == null ? this.SaveFileAs(xc) : xc.Save(Settings.Instance.xw_settings);
		}

		///////////////////////////////////////////////////////
		/// 
		public bool SaveFileAs(xigt_corpus xc = null)
		{
			if ((xc ?? (xc = corpora.SelectedItem)) == null)
				return false;

			var dlg = new CommonSaveFileDialog
			{
				Title = String.Format("Save {0} as...", xc.CorpusName),
				InitialDirectory = xc.Directory ?? Settings.Instance.LastDirectory,
				DefaultFileName = xc.CorpusName,
				DefaultExtension = ".xml",
				EnsurePathExists = true,
				OverwritePrompt = true,

				Filters =
				{
					new CommonFileDialogFilter("Xigt-XML",".xml"),
					new CommonFileDialogFilter("All files",".*")
				}
			};

			if (dlg.ShowDialog(this) != CommonFileDialogResult.Ok)
				return false;

			var fn = new Uri(Path.GetFullPath(dlg.FileName)).LocalPath;

			if (!check_overwrite_file(fn))
				return false;

			xc.FullPath = fn;
			xc.Save(Settings.Instance.xw_settings);
			Settings.Instance.SetRecentFile(fn);
			return true;
		}

		///////////////////////////////////////////////////////
		/// 
		bool check_overwrite_file(String fn)
		{
			xigt_corpus xc_cur;

			if (!File.Exists(fn) || (xc_cur = corpora.FindCorpus(fn)) == null)
				return true;

			if (xc_cur.Modified && MessageBox.Show(String.Format(
@"The file you are attempting to overwrite:

'{0}'

is open in the editor and has unsaved changes.

Do you want to discard changes and close the file?", fn), "Overwrite conflict", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.No)
				return false;

			Settings.Instance.RemoveRecentFile(xc_cur.FullPath);
			corpora.Remove(xc_cur);
			return true;
		}

		///////////////////////////////////////////////////////
		/// 
		MessageBoxResult query_save_file(xigt_corpus xc)
		{
			MessageBoxResult r;
			do
				if (!xc.NeedSave)
					return MessageBoxResult.Yes;
			while ((r = MessageBox.Show(
							String.Format("Save changes to {0}?", xc.CorpusName),
							"Unsaved changes",
							MessageBoxButton.YesNoCancel,
							MessageBoxImage.Question)) == MessageBoxResult.Yes && !SaveFile(xc));
			return r;
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_close(Object sender, RoutedEventArgs e) { CloseFile(); }
		public bool CloseFile(xigt_corpus xc = null)
		{
			if ((xc ?? (xc = corpora.SelectedItem)) != null)
			{
				if (query_save_file(xc) == MessageBoxResult.Cancel)
					return false;

				corpora.Remove(xc);
			}
			return true;
		}

		///////////////////////////////////////////////////////
		/// 
		public bool CloseAllFiles(bool dont_remove_saved = false)
		{
			var rg = corpora;
			int c = rg.Count;
			while (--c >= 0)
			{
				switch (query_save_file(rg[c]))
				{
					case MessageBoxResult.Cancel:
						return false;
					case MessageBoxResult.Yes:
						if (dont_remove_saved)
							continue;
						break;
				}
				corpora.Remove(rg[c]);
			}
			return true;
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_delete(Object sender, RoutedEventArgs e) { DeleteFile(); }
		public void DeleteFile()
		{
		}

		///////////////////////////////////////////////////////
		/// 
		private void corpus_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.Insert:
					InsertIgt(w_corpus.SelectedIndex);
					break;

				case Key.Delete:
					DeleteIgt(w_corpus.SelectedIndex);
					break;

				default:
					return;
			}
			e.Handled = true;
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_igt_add(Object sender, RoutedEventArgs e) { InsertIgt(-1); }
		void mw_click_igt_ins(Object sender, RoutedEventArgs e) { InsertIgt(w_corpus.SelectedIndex); }
		void InsertIgt(int ix)
		{
			//xigt_corpus xc;
			//if ((xc = corpora.SelectedItem) != null)
			//{
			//	xc.SelectedItem = xc.InsertIgt(ix);
			//	w_corpus.FocusSelectedItem();
			//}

			xigt_corpus xc;
			if ((xc = (xigt_corpus)w_corpora.SelectedItem) != null)
			{
				xc.SelectedItem = xc.InsertIgt(ix);
				w_corpus.FocusSelectedItem();
			}
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_igt_del(Object sender, RoutedEventArgs e) { DeleteIgt(w_corpus.SelectedIndex); }
		void DeleteIgt(int ix)
		{
			xigt_corpus xc;
			if ((xc = (xigt_corpus)w_corpora.SelectedItem) != null && (uint)ix < (uint)xc.Count)
			{
				xc.Remove(xc[ix]);
				w_corpus.SelectedIndex = ix;
				w_corpus.FocusSelectedItem();
			}


			//xigt_corpus xc;
			//if ((xc = corpora.SelectedItem) == null)
			//	return;

			//throw not.tested;

			//IList _del;
			//int ix, c;
			//if ((c = (_del = w_corpus.SelectedItems).Count) == 0)
			//{
			//	throw not.impl;
			//}
			//else if (c == xc.Count)
			//{
			//	xc.Clear();
			//}
			//else if (c > 1)
			//{
			//	throw not.impl;
			//}
			//else if ((ix = w_corpus.SelectedIndex) != -1)	/* c==1 */
			//{
			//	xc.RemoveAt(ix);
			//	if (ix == xc.Count)
			//		ix--;
			//	w_corpus.SelectedIndex = ix;
			//}
			//else
			//{
			//	throw not.expected;
			//}
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_igt_up(Object sender, RoutedEventArgs e)
		{
			//int ix;
			//xigt_corpus xc;
			//if ((xc = corpora.SelectedItem) != null && (ix = w_corpus.SelectedIndex) > 0)
			//	xc.Move(ix, ix - 1);
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_igt_down(Object sender, RoutedEventArgs e)
		{
			//int ix;
			//xigt_corpus xc;
			//if ((xc = corpora.SelectedItem) != null && (ix = w_corpus.SelectedIndex) >= 0 && ix < xc.Count - 1)
			//	xc.Move(ix, ix + 1);
		}

		///////////////////////////////////////////////////////
		/// 
		private void igt_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.Insert:
					InsertTier(w_igt.SelectedIndex);
					break;

				case Key.Delete:
					DeleteTier(w_igt.SelectedIndex);
					break;

				default:
					return;
			}
			e.Handled = true;
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_tier_add(Object sender, RoutedEventArgs e) { InsertTier(-1); }
		void mw_click_tier_ins(Object sender, RoutedEventArgs e) { InsertTier(w_igt.SelectedIndex); }
		void InsertTier(int ix)
		{
			igt igt;
			if ((igt = (igt)w_corpus.SelectedItem) != null)
			{
				igt.SelectedItem = igt.InsertTier(ix);
				w_igt.FocusSelectedItem();
			}
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_tier_del(Object sender, RoutedEventArgs e) { DeleteTier(w_igt.SelectedIndex); }
		void DeleteTier(int ix)
		{
			igt igt;
			if ((igt = (igt)w_corpus.SelectedItem) != null && (uint)ix < (uint)igt.Count)
			{
				igt.Remove(igt[ix]);
				w_igt.SelectedIndex = ix;
				w_igt.FocusSelectedItem();
			}
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_tier_up(Object sender, RoutedEventArgs e)
		{
			//int ix;
			//igt igt;
			//if ((igt = SelectedIgt) != null && (ix = w_igt.SelectedIndex) > 0)
			//	igt.Move(ix, ix - 1);
		}

		///////////////////////////////////////////////////////
		/// 
		void mw_click_tier_down(Object sender, RoutedEventArgs e)
		{
			//int ix;
			//igt igt;
			//if ((igt = SelectedIgt) != null && (ix = w_igt.SelectedIndex) >= 0 && ix < igt.Count - 1)
			//	igt.Move(ix, ix + 1);
		}
	};
}
