﻿using System;
using System.Globalization;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Markup;
using System.Windows.Shapes;

using alib;
using alib.Enumerable;
using alib.Debugging;
using alib.Wpf;

using xigt.ui;

namespace xigt.app
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class BooleanTo90Converter : IValueConverter
	{
		public static readonly IValueConverter Instance = new BooleanTo90Converter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _) { return (bool)v ? 90.0 : 0.0; }

		Object IValueConverter.ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class TextRefToHilightConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new TextRefToHilightConverter();

		static Brush dim_brush = new SolidColorBrush(Color.FromArgb(0x70, 0xFF, 0xFF, 0xFF));

		public Object Convert(Object v, Type T, Object to_check, CultureInfo _)
		{
#if TREF
			item_ctrl _i;
			if ((_i = to_check as item_ctrl) == null)
				return dim_brush;

			var hov = (RefExpr)v;
			if (hov.TargetItem == null)
				return dim_brush;

			return hov.TargetItem == _i.DataContext ? Brushes.Yellow : dim_brush;
#else
			return Brushes.Yellow;
#endif
		}

		Object IValueConverter.ConvertBack(Object value, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class IsReadOnlyToBrushConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new IsReadOnlyToBrushConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			var is_read_only = (bool)v;

			Color c;
			String sc;

			if ((sc = param as String) != null)
				c = sc.ToColor();
			else if (param is Color)
				c = (Color)param;
			else if (param is HSL)
				c = ((HSL)param).ToRGB();
			else
				throw new InvalidOperationException();

			if (!is_read_only)
			{
				var hsl = new HSL(c);
				hsl.S *= .90;
				hsl.L *= 1.08;
				c = hsl.ToRGB();
			}
			return new SolidColorBrush(c);
		}

		Object IValueConverter.ConvertBack(Object value, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	public sealed class BooleanToHilightConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new BooleanToHilightConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			return (bool)v ? Brushes.DarkOrange : null;
		}

		Object IValueConverter.ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};
#if false
	public sealed class NonNullRefExprToHilightConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new NonNullRefExprToHilightConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			return ((RefExpr)v).TargetItem != null ? Brushes.Yellow : null;
		}

		Object IValueConverter.ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};
#endif

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class TierBrushConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new TierBrushConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			Color c;
			var type = (String)v;
			if (type == null || !Settings.Instance.TierColors.TryGetValue(type, out c))
				return Brushes.White;

			return SolidColorBrushCache.Get(c);
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class TierOrientConv : IValueConverter
	{
		public static IValueConverter Instance = new TierOrientConv();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			if (v == null)
				return Orientation.Horizontal;

			return (Orientation)(TierOrientation)v;
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _)
		{
			return (TierOrientation)(Orientation)v;
		}
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class TierOrientToBooleanConverter : IValueConverter
	{
		public static IValueConverter Instance = new TierOrientToBooleanConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			return (TierOrientation)v == TierOrientation.Horizontal;
		}

		public Object ConvertBack(Object value, Type T, Object param, CultureInfo _)
		{
			return (bool)value ? TierOrientation.Horizontal : TierOrientation.Vertical;
		}
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class TierModeConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new TierModeConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			switch ((tier_mode)v)
			{
				case tier_mode.Source:
					return "src";
				case tier_mode.Align:
					return "align";
				case tier_mode.Alias:
					return "alias";
				case tier_mode.Segmentation:
					return "seg";
				case tier_mode.Annotation:
					return "annot";
				case tier_mode.Dependency:
					return "dep";
				case tier_mode.BilingualAlignment:
					return "bitext";
				case tier_mode.PhraseStructure:
					return "ps";
				default:
					throw not.valid;
			}
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class ItemModeConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new ItemModeConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			switch ((item_mode)v)
			{
				case item_mode.Source:
					return "src";
				case item_mode.Align:
					return "align";
				case item_mode.Alias:
					return "alias";
				case item_mode.Segmentation:
					return "seg";
				case item_mode.Annotation:
					return "annot";
				case item_mode.Dependency:
					return "dep";
				case item_mode.BilingualAlignment:
					return "bitext";
				case item_mode.PSNode:
					return "ps-node";
				default:
					throw not.valid;
			}
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class ContentRefVisibilityConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new ContentRefVisibilityConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			switch ((item_mode)v)
			{
				case item_mode.Alias:
				case item_mode.Annotation:
				case item_mode.Segmentation:
				case item_mode.Dependency:
					return Visibility.Visible;
			}
			return Visibility.Collapsed;
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public sealed class AlignRefVisibilityConverter : IValueConverter
	{
		public static readonly IValueConverter Instance = new AlignRefVisibilityConverter();

		public Object Convert(Object v, Type T, Object param, CultureInfo _)
		{
			switch ((item_mode)v)
			{
				case item_mode.Align:
				case item_mode.Annotation:
					return Visibility.Visible;
			}
			return Visibility.Collapsed;
		}

		public Object ConvertBack(Object v, Type T, Object param, CultureInfo _) { throw not.impl; }
	};
}