﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Controls.Primitives;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Markup;
using System.Windows.Media.Imaging;
using System.Xaml;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.String;
using alib.Wpf;

using Microsoft.Win32;
using Microsoft.WindowsAPICodePack.Dialogs;

namespace xigt.app
{
	using Path = System.IO.Path;
	using xigt.ui;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public partial class main : Window
	{
		public static readonly DependencyProperty EnableTierBrushesProperty =
			DependencyProperty.Register("EnableTierBrushes", typeof(bool), typeof(main), new PropertyMetadata(true));

		public bool EnableTierBrushes
		{
			get { return (bool)GetValue(EnableTierBrushesProperty); }
			set { SetValue(EnableTierBrushesProperty, value); }
		}

		public main()
		{
#if false
			EventManager.RegisterClassHandler(typeof(main), Selector.UnselectedEvent, new RoutedEventHandler((o, e) =>
				{
					var el = (UIElement)e.OriginalSource;
					var fm = FocusManager.GetFocusScope(el);
					var kf = Keyboard.FocusedElement;
					Debug.Print("Unselected: kbd: {0}   logical: Scope={1} Element={2}",
						kf != null ? kf.GetType().Name + "-" + kf.GetHashCode().ToString("X8") : "null",
						fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
						el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
					Debug.Print("{0}", el);
				}), true);
#endif

#if false
			EventManager.RegisterClassHandler(typeof(main), FocusManager.GotFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				var kf = Keyboard.FocusedElement;
				Debug.Print("GotFocus: kbd: {0}   logical: Scope={1} Element={2}",
					kf != null ? kf.GetType().Name + "-" + kf.GetHashCode().ToString("X8") : "null",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);
			EventManager.RegisterClassHandler(typeof(main), FocusManager.LostFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				var kf = Keyboard.FocusedElement;
				Debug.Print("LostFocus: kbd: {0}   logical: Scope={1} Element={2}",
					kf != null ? kf.GetType().Name + "-" + kf.GetHashCode().ToString("X8") : "null",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);
#endif

#if false
			EventManager.RegisterClassHandler(typeof(main), Keyboard.PreviewGotKeyboardFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				Debug.Print("PreviewGotKeyboardFocus: Scope={0} Element={1}",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);
			EventManager.RegisterClassHandler(typeof(main), Keyboard.PreviewLostKeyboardFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				Debug.Print("PreviewLostKeyboardFocus: Scope={0} Element={1}",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);

			EventManager.RegisterClassHandler(typeof(main), Keyboard.GotKeyboardFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				Debug.Print("GotKeyboardFocus: Scope={0} Element={1}",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);
			EventManager.RegisterClassHandler(typeof(main), Keyboard.LostKeyboardFocusEvent, new RoutedEventHandler((o, e) =>
			{
				var el = (UIElement)e.OriginalSource;
				var fm = FocusManager.GetFocusScope(el);
				Debug.Print("LostKeyboardFocus: Scope={0} Element={1}",
					fm != null ? fm.GetType().Name + "-" + fm.GetHashCode().ToString("X8") : "null",
					el != null ? el.GetType().Name + "-" + el.GetHashCode().ToString("X8") : "null");
			}), true);
#endif
			InitializeComponent();

			Loaded += OnLoaded;
		}

		public corpora corpora { get { return base.DataContext as corpora; } }

		void OnLoaded(Object o, RoutedEventArgs e)
		{
			Loaded -= OnLoaded;

			StartupAction a;

			switch (a = Settings.Instance.StartupAction)
			{
				case StartupAction.ReloadLastSessionOrNone:
				case StartupAction.ReloadLastSessionOrNew:
					if (!LoadFiles(Settings.Instance.SessionFiles) && a == StartupAction.New)
						goto new_corpus;
					break;

				case StartupAction.ReloadLastFileOrNone:
				case StartupAction.ReloadLastFileOrNew:
					if (!LoadFile(Settings.Instance.SelectedCorpusFilename) && a == StartupAction.New)
						goto new_corpus;
					break;

				case StartupAction.New:
				new_corpus:
					NewCorpus();
					break;
			}

			w_corpora.SelectedIndex = Settings.Instance.SelectedCorpusIndex;
			w_corpora.FocusSelectedItem();
		}

		public bool f_force_discard;

		protected override void OnClosing(CancelEventArgs e)
		{
			if (FocusManager.GetFocusScope((DependencyObject)Keyboard.Focus(this)) != this ||
				(!f_force_discard && Settings.Instance.SaveOnExit && !CloseAllFiles(true)))
			{
				e.Cancel = true;
				return;
			}

			base.OnClosing(e);
		}

		protected override void OnClosed(EventArgs e)
		{
			Settings.Instance.SessionFiles = corpora
							.Select(c => c.FullPath)
							.Where(fn => !String.IsNullOrEmpty(fn))
							.ToArray();

			Settings.Instance.SelectedCorpusIndex = w_corpora.SelectedIndex;

			base.OnClosed(e);
		}

		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e);

			if (e.Property == TextBlock.FontFamilyProperty)
			{
				var d = ((FontFamily)e.NewValue).FamilyNames.Values;
				foreach (var mi in w_menu.mi_font_families.Items.OfType<MenuItem>())
				{
					mi.IsChecked = d.Contains((String)mi.Header, StringComparer.InvariantCultureIgnoreCase);
				}
			}
			else if (e.Property == NameScope.NameScopeProperty)
			{
				((INameScope)e.NewValue).RegisterName("S", Settings.Instance);
			}
		}

		void w_find_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Enter)
			{
				var s = w_find.Text.Trim();
				Debug.Print("search {0}", w_find.Text);
			}
		}

		void Splitter1_dblclk(object sender, MouseButtonEventArgs e)
		{
			w_grid.ColumnDefinitions[0].Width = GridLength.Auto;
		}
		void Splitter2_dblclk(object sender, MouseButtonEventArgs e)
		{
			w_grid.ColumnDefinitions[2].Width = GridLength.Auto;
		}

		private void Button_LostMouseCapture(object sender, MouseEventArgs e)
		{
			Keyboard.Focus((ButtonBase)e.OriginalSource);
		}
	};
}
