﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.Wpf;

using xigt.app;

namespace xigt.ui
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public partial class tier_ctrl : Border
	{
		static readonly DependencyPropertyKey ContentTypeLabelPropertyKey =
			DependencyProperty.RegisterReadOnly("ContentTypeLabel", typeof(String), typeof(tier_ctrl), new FrameworkPropertyMetadata(null));

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public String ContentTypeLabel
		{
			[DebuggerStepThrough]
			get { return (String)GetValue(ContentTypeLabelPropertyKey.DependencyProperty); }
		}

		public static readonly DependencyProperty XmlDisplayProperty =
			DependencyProperty.Register("XmlDisplay", typeof(bool), typeof(tier_ctrl), new PropertyMetadata(false));

		public bool XmlDisplay
		{
			get { return (bool)GetValue(XmlDisplayProperty); }
			set { SetValue(XmlDisplayProperty, value); }
		}

		public tier_ctrl()
		{
			InitializeComponent();
		}

		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e);

			tier t;
			if (e.Property == DataContextProperty && (t = e.NewValue as tier) != null)
			{
				//SetValue(ShowAlignRefPropertyKey, t != null && t.alignment != null && t.alignment != t.content);
				//SetValue(ShowContentRefPropertyKey, t != null && t.content != null);
				SetValue(ContentTypeLabelPropertyKey, t != null && t.content == t.alignment ? "seg" : "content");
			}
		}

		protected override void OnPreviewMouseLeftButtonDown(MouseButtonEventArgs e)
		{
			var t = (tier)DataContext;
			if (t != null)
				t.Host.SelectedItem = t;
		}
	};


	public sealed class TierContentTemplateSelector : DataTemplateSelector
	{
		public static readonly TierContentTemplateSelector Instance = new TierContentTemplateSelector();

		public override DataTemplate SelectTemplate(Object item, DependencyObject _cp)
		{
			tier t;
			if ((t = item as tier) == null)
				return null;

			var cp = (ContentPresenter)_cp;
			var cc = (ContentControl)cp.TemplatedParent;

			tier_mode m;

			switch (t.Mode)
			{
				case tier_mode.Dependency:
				case tier_mode.BilingualAlignment:
				case tier_mode.PhraseStructure:
					m = t.Mode;
					break;
				default:
					m = tier_mode.Source;
					break;
			}
			return (DataTemplate)cc.Resources[m];
		}
	};
}


namespace xigt.ui
{
	using ICSharpCode.AvalonEdit;
	using ICSharpCode.AvalonEdit.Document;
	using ICSharpCode.AvalonEdit.Highlighting;

	public class XmlEditor : TextEditor
	{
		new public tier DataContext { get { return base.DataContext as tier; } }

		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e);

			if (e.Property == DataContextProperty)
			{
				if (IsModified)
					throw not.valid;

				Text = DataContext != null ? DataContext.FormattedXml : String.Empty;
				IsModified = false;
			}
		}

		protected override void OnPreviewLostKeyboardFocus(KeyboardFocusChangedEventArgs e)
		{
			/* nop: base.OnPreviewLostKeyboardFocus(e); */

			if (!this.IsModified)
				return;

			this.IsModified = false;
			if (DataContext.TrySetRawXml(this.Text))
				return;

			if (MessageBox.Show(@"There is an error in the XML.

Do you want to abandon your changes?", "Invalid Xigt-XML text", MessageBoxButton.YesNo, MessageBoxImage.Error) == MessageBoxResult.No)
			{
				this.IsModified = true;
				e.Handled = true;
			}
			else
			{
				Text = DataContext.FormattedXml;
				this.IsModified = false;
			}
		}
	};
}

