﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Reflection;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;

using alib;
using alib.Math;
using alib.Debugging;
using alib.Enumerable;
using alib.Wpf;

namespace xigt.ui
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class tier_items_ui : ItemsControl
	{
		public static readonly DependencyProperty OrientationProperty =
			StackPanel.OrientationProperty.AddOwner(typeof(tier_items_ui), new FrameworkPropertyMetadata((o, e) => ((tier_items_ui)o).ResetDims()));

		public Orientation Orientation
		{
			get { return (Orientation)GetValue(OrientationProperty); }
			set { SetValue(OrientationProperty, value); }
		}

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		static readonly DependencyPropertyKey
			Width0PropertyKey = DependencyProperty.RegisterReadOnly("Width0", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height0PropertyKey = DependencyProperty.RegisterReadOnly("Height0", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0)),

			Width1PropertyKey = DependencyProperty.RegisterReadOnly("Width1", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height1PropertyKey = DependencyProperty.RegisterReadOnly("Height1", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0)),

			Width2PropertyKey = DependencyProperty.RegisterReadOnly("Width2", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height2PropertyKey = DependencyProperty.RegisterReadOnly("Height2", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0)),

			Width3PropertyKey = DependencyProperty.RegisterReadOnly("Width3", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height3PropertyKey = DependencyProperty.RegisterReadOnly("Height3", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0)),

			Width4PropertyKey = DependencyProperty.RegisterReadOnly("Width4", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height4PropertyKey = DependencyProperty.RegisterReadOnly("Height4", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0)),

			Width5PropertyKey = DependencyProperty.RegisterReadOnly("Width5", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(20.0)),
			Height5PropertyKey = DependencyProperty.RegisterReadOnly("Height5", typeof(Double), typeof(tier_items_ui), new FrameworkPropertyMetadata(0.0));


		public Double Width0 { get { return (Double)GetValue(Width0PropertyKey.DependencyProperty); } }
		public Double Height0 { get { return (Double)GetValue(Height0PropertyKey.DependencyProperty); } }

		public Double Width1 { get { return (Double)GetValue(Width1PropertyKey.DependencyProperty); } }
		public Double Height1 { get { return (Double)GetValue(Height1PropertyKey.DependencyProperty); } }

		public Double Width2 { get { return (Double)GetValue(Width2PropertyKey.DependencyProperty); } }
		public Double Height2 { get { return (Double)GetValue(Height2PropertyKey.DependencyProperty); } }

		public Double Width3 { get { return (Double)GetValue(Width3PropertyKey.DependencyProperty); } }
		public Double Height3 { get { return (Double)GetValue(Height3PropertyKey.DependencyProperty); } }

		public Double Width4 { get { return (Double)GetValue(Width4PropertyKey.DependencyProperty); } }
		public Double Height4 { get { return (Double)GetValue(Height4PropertyKey.DependencyProperty); } }

		public Double Width5 { get { return (Double)GetValue(Width5PropertyKey.DependencyProperty); } }
		public Double Height5 { get { return (Double)GetValue(Height5PropertyKey.DependencyProperty); } }

		protected void ResetDims()
		{
			ClearValue(Width0PropertyKey);
			ClearValue(Height0PropertyKey);

			ClearValue(Width1PropertyKey);
			ClearValue(Height1PropertyKey);

			ClearValue(Width2PropertyKey);
			ClearValue(Height2PropertyKey);

			ClearValue(Width3PropertyKey);
			ClearValue(Height3PropertyKey);

			ClearValue(Width4PropertyKey);
			ClearValue(Height4PropertyKey);

			ClearValue(Width5PropertyKey);
			ClearValue(Height5PropertyKey);
		}

		protected void UpdateDims(item_ctrl _it)
		{
			Double d;

			if (Orientation == Orientation.Vertical)
			{
				if ((d = _it.w_id.ActualWidth) > Width0)
					SetValue(Width0PropertyKey, d);
				if ((d = _it.w_line.ActualWidth) > Width1)
					SetValue(Width1PropertyKey, d);
				if ((d = _it.w_tag.ActualWidth) > Width2)
					SetValue(Width2PropertyKey, d);
				if ((d = _it.w_text.ActualWidth) > Width3)
					SetValue(Width3PropertyKey, d);
				if (_it.w_content.IsVisible && (d = _it.w_content.ActualWidth) > Width4)
					SetValue(Width4PropertyKey, d);
				if (_it.w_alignment.IsVisible && (d = _it.w_alignment.ActualWidth) > Width5)
					SetValue(Width5PropertyKey, d);
			}
			else
			{
				if ((d = _it.w_id.ActualHeight) > Height0)
					SetValue(Height0PropertyKey, d);
				if ((d = _it.w_line.ActualHeight) > Height1)
					SetValue(Height1PropertyKey, d);
				if ((d = _it.w_tag.ActualHeight) > Height2)
					SetValue(Height2PropertyKey, d);
				if ((d = _it.w_text.ActualHeight) > Height3)
					SetValue(Height3PropertyKey, d);
				if (_it.w_content.IsVisible && (d = _it.w_content.ActualHeight) > Height4)
					SetValue(Height4PropertyKey, d);
				if (_it.w_alignment.IsVisible && (d = _it.w_alignment.ActualHeight) > Height5)
					SetValue(Height5PropertyKey, d);
			}
		}
	};


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public partial class tier_items_ctrl : tier_items_ui
	{
		public tier_items_ctrl()
		{
			InitializeComponent();
		}

		void item_size_changed(Object cp, SizeChangedEventArgs e)
		{
			UpdateDims((item_ctrl)VisualTreeHelper.GetChild((ContentPresenter)cp, 0));
		}

		protected override void OnItemsChanged(NotifyCollectionChangedEventArgs e)
		{
			ResetDims();
		}

		protected override void PrepareContainerForItemOverride(DependencyObject cp, Object item)
		{
			((ContentPresenter)cp).SizeChanged += item_size_changed;

			base.PrepareContainerForItemOverride(cp, item);
		}

		protected override void ClearContainerForItemOverride(DependencyObject cp, Object item)
		{
			base.ClearContainerForItemOverride(cp, item);

			((ContentPresenter)cp).SizeChanged -= item_size_changed;
		}
	};
}
