﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.Wpf;
using alib.Character;

namespace xigt.ui
{
	public partial class bitext_items : Grid
	{
		public bitext_items()
		{
			InitializeComponent();
		}

		const Double pen_thick = 1.3;
		const Double th2 = pen_thick / 2.0;

		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			base.OnPropertyChanged(e);

			if (e.Property == DataContextProperty)
			{
				var dt = (DataTemplate)Resources["dt_item"];

				w_src_items.Children.Clear();
				w_src_items.ColumnDefinitions.Clear();
				w_tgt_items.Children.Clear();
				w_tgt_items.ColumnDefinitions.Clear();

				tier tier;
				if ((tier = DataContext as tier) != null)
				{
					FrameworkElement el;

					foreach (var ch in tier.source)
					{
						el = (FrameworkElement)dt.LoadContent();
						el.DataContext = ch;
						Grid.SetColumn(el, w_src_items.ColumnDefinitions.Count);
						if (w_src_items.ColumnDefinitions.Count > 0)
							el.Margin = new Thickness(20, 0, 0, 0);
						w_src_items.ColumnDefinitions.Add(new ColumnDefinition());
						w_src_items.Children.Add(el);
					}

					foreach (var ch in tier.target)
					{
						el = (FrameworkElement)dt.LoadContent();
						el.DataContext = ch;
						Grid.SetColumn(el, w_tgt_items.ColumnDefinitions.Count);
						if (w_tgt_items.ColumnDefinitions.Count > 0)
							el.Margin = new Thickness(20, 0, 0, 0);
						w_tgt_items.ColumnDefinitions.Add(new ColumnDefinition());
						w_tgt_items.Children.Add(el);
					}

					foreach (var item in tier)
					{
						el = new Border
						{
							DataContext = item,
							BorderBrush = Brushes.Black,
							BorderThickness = new Thickness(1),
							HorizontalAlignment = HorizontalAlignment.Left,
							VerticalAlignment = VerticalAlignment.Top,
							Background = Brushes.White,
							Child = new TextBlock
							{
								Text = item.id,
								Margin = new Thickness(2, 0, 2, 0),
							},
						};
						Grid.SetRow(el, 1);
						Grid.SetColumn(el, 1);
						Children.Add(el);
					}
				}
			}
		}


		protected override Size ArrangeOverride(Size sz)
		{
			sz = base.ArrangeOverride(sz);

			tier tier;
			if ((tier = DataContext as tier) != null)
			{
				var igt = tier.GetIgt();
				IElem iel;
				item item, e_src, e_tgt;

				foreach (FrameworkElement el in InternalChildren)
				{
					if ((item = el.DataContext as item) == null)
						continue;

					if (!el.IsMeasureValid)
						Nop.X();

					foreach (var ss in item.source.Split(Charset.comma_semi_ws, StringSplitOptions.RemoveEmptyEntries))
					{
						foreach (var tt in item.target.Split(Charset.comma_semi_ws, StringSplitOptions.RemoveEmptyEntries))
						{
							if (!igt.igt_names.TryGetValue(ss, out iel) || (e_src = iel as item) == null)
								continue;

							if (!igt.igt_names.TryGetValue(tt, out iel) || (e_tgt = iel as item) == null)
								continue;

							var fe_src = w_src_items.Children.OfType<FrameworkElement>().First(x => x.DataContext == e_src);
							var fe_tgt = w_tgt_items.Children.OfType<FrameworkElement>().First(x => x.DataContext == e_tgt);

							var pt1 = fe_src.TransformToAncestor(this).Transform(new Point(fe_src.ActualWidth / 2 - th2, fe_src.ActualHeight - th2));
							var pt2 = fe_tgt.TransformToAncestor(this).Transform(new Point(fe_tgt.ActualWidth / 2 - th2, th2));

							var pt = new Rect(pt1, pt2).Center();
							pt.X -= el.DesiredSize.Width / 2;
							pt.Y -= el.DesiredSize.Height / 2;
							el.Arrange(new Rect(pt, el.DesiredSize));
						}
					}
				}
			}

			return sz;
		}

		protected override void OnRender(DrawingContext dc)
		{
			base.OnRender(dc);

			tier tier;
			if ((tier = DataContext as tier) != null)
			{
				IElem iel;
				item item, e_src, e_tgt;
				var pen = new Pen(Brushes.Black, pen_thick);
				var igt = tier.GetIgt();

				foreach (FrameworkElement el in InternalChildren)
				{
					if ((item = el.DataContext as item) == null)
						continue;

					foreach (var ss in item.source.Split(Charset.comma_semi_ws, StringSplitOptions.RemoveEmptyEntries))
					{
						foreach (var tt in item.target.Split(Charset.comma_semi_ws, StringSplitOptions.RemoveEmptyEntries))
						{
							if (!igt.igt_names.TryGetValue(ss, out iel) || (e_src = iel as item) == null)
								continue;

							if (!igt.igt_names.TryGetValue(tt, out iel) || (e_tgt = iel as item) == null)
								continue;

							var fe_src = w_src_items.Children.OfType<FrameworkElement>().First(x => x.DataContext == e_src);
							var fe_tgt = w_tgt_items.Children.OfType<FrameworkElement>().First(x => x.DataContext == e_tgt);

							var pt1 = fe_src.TransformToAncestor(this).Transform(new Point(fe_src.ActualWidth / 2 - th2, fe_src.ActualHeight - th2));
							var pt2 = fe_tgt.TransformToAncestor(this).Transform(new Point(fe_tgt.ActualWidth / 2 - th2, th2));

							//Debug.Print("{0} {1}", pt1, pt2);
							//Debug.Print("{0:X8} {1} src=[{2}] tgt=[{3}]", this.GetHashCode(), item.id, item.source, item.target);
							//Debug.Print("{0} {1}", VisualTreeHelper.GetOffset(fe_src), VisualTreeHelper.GetOffset(fe_tgt));
							//Debug.Print("{0} {1}", VisualTreeHelper.GetContentBounds(fe_src), VisualTreeHelper.GetContentBounds(fe_tgt));

							dc.DrawLine(pen, pt1, pt2);
						}
					}
				}
			}
		}
	};
}
