﻿using System;
using System.Threading;
using System.Threading.Tasks;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Xml;

using System.Windows;
using alib.Debugging;
using alib.Enumerable;

namespace xigt
{
	public enum AddCorpusResult
	{
		Ok,
		AlreadyLoaded,
		FileNotFound,
	};

	public enum item_mode
	{
		Unknown /*				*/ = 0,
		Source /*				*/ = 1,
		Align /*				*/ = 2,
		Alias /*				*/ = 3,
		Segmentation /*			*/ = 4,
		Annotation /*			*/ = 5,
		Dependency /*			*/ = 6,
		BilingualAlignment /*	*/ = 7,
		PSLeaf /*				*/ = Annotation,
		PSNode /*				*/ = 9,
		PSExtra /*				*/ = 10,
	};

	public enum TierOrientation
	{
		Horizontal =/*		*/ 0,
		Vertical = /*		*/ 1,
		Default = /*		*/ 2,
		//NotSet = /*			*/ 4,
	};

	public enum tier_mode
	{
		Unknown /*				*/ = 0,
		Source /*				*/ = 1,
		Align /*				*/ = 2,
		Alias /*				*/ = 3,
		Segmentation /*			*/ = 4,
		Annotation /*			*/ = 5,
		Dependency /*			*/ = 6,
		BilingualAlignment /*	*/ = 7,
		PhraseStructure /*		*/ = 8,
	};

	public interface IEntity : INotifyPropertyChanged
	{
		String id { get; set; }
		String type { get; set; }
	};

	public interface IElem : IEntity
	{
		IHost Host { get; }
		//bool IsFirst { get; }
		//bool IsLast { get; }
	};

	public interface IElem<out H> : IElem
		where H : class, IHost
	{
		new H Host { get; }
	};

	public interface IHost : IEntity, IList, INotifyCollectionChanged
	{
	};

	public interface IHost<out T> : IHost
		where T : class, IElem
	{
		T SelectedItem { get; }
		void Move(int oldIndex, int newIndex);
	};

	public interface IElemHost<out H, out T> : IElem<H>, IHost<T>
		where H : class, IHost<IElem<H>>
		where T : class, IElem<IHost<T>>
	{
	};

	//public interface IMetadataHost : IHost
	//{
	//	metadata Metadata { get; }
	//	bool HasMetadata { get; }
	//};


	public static class _isb_ext
	{
		public static igt GetIgt(this IElem _this)
		{
			igt igt;
			var cur = _this;
			while ((igt = cur as igt) == null)
				cur = (IElem)cur.Host;
			return igt;
		}
		public static xigt_corpus GetCorpus(this IEntity _this)
		{
			xigt_corpus xc;
			var cur = _this as IElem;
			while ((xc = cur as xigt_corpus) == null && cur != null)
				cur = cur.Host as IElem;
			return xc;
		}
	};
}
