﻿using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

using alib;
using alib.Enumerable;
using alib.Debugging;

namespace xigt
{
	public class XigtXmlReader : Dictionary<String, _entity>
	{
		static readonly Char[] rgch_standoff_split = new[] { '[', ']', ':', ' ' };

		public xigt_corpus read(String fn)
		{
			var corpus = new xigt_corpus { Filename = fn = System.IO.Path.GetFullPath(fn) };

			var xd = new XmlDocument();

			xd.Load(XmlReader.Create(fn, new XmlReaderSettings
			{
				IgnoreComments = true
			}));

			_read_corpus(corpus, (XmlElement)xd.ChildNodes[1]);

			return corpus;
		}

		public void _read_metadata(IMetadataHost mh, XmlElement e_meta)
		{
			//String s_attr;

			if (mh is IHost && ((IHost)mh).GetList().Count > 0)
				throw new Exception(String.Format("<metadata> must appear before all <{0}> elements"));

			foreach (XmlElement e in e_meta.ChildNodes)
			{
				if (e.Name != "meta")
					throw not.expected;

				mh.Metadata.Add(new meta
				{
					type = e.GetAttribute("type"),
				});

#if TEMP || true
				var igt = mh as igt;
				if (igt != null)
				{
					String s_attr;
					if ((s_attr = e.GetAttribute("doc-id")).Length > 0)
						igt.DocId = s_attr;
					else if ((s_attr = e.GetAttribute("iso-639-3")).Length > 0)
					{
						if (igt.Language == null)
							igt.Language = s_attr;
						else
							igt.Language += "; " + s_attr;
					}
				}
#endif

				//if ((s_attr = e.GetAttribute("doc-id")).Length > 0)
				//	mh.DocId = s_attr;
				//else if ((s_attr = e.GetAttribute("iso-639-3")).Length > 0)
				//{
				//	if (mh.Language == null)
				//		mh.Language = s_attr;
				//	else
				//		mh.Language += "; " + s_attr;
				//}
			}
		}

		public void _read_corpus(xigt_corpus corpus, XmlElement e_corpus)
		{
			if (e_corpus.Name != "xigt-corpus")
				throw not.expected;

			foreach (XmlElement e in e_corpus.ChildNodes)
			{
				switch (e.Name)
				{
					case "metadata":
						_read_metadata(corpus, e);
						break;

					case "igt":
						base.Clear();
						_read_igt(corpus, e);
						break;

					default:
						throw new Exception("unknown tag " + e.Name);
				}
			}
		}

		public void _read_igt(xigt_corpus corpus, XmlElement e_igt)
		{
			var igt = new igt
			{
				id = e_igt.GetAttribute("id"),
				type = e_igt.GetAttribute("type"),
			};
			corpus.Add(igt);

			foreach (XmlElement e in e_igt.ChildNodes)
			{
				switch (e.Name)
				{
					case "metadata":
						_read_metadata(igt, e);
						break;

					case "tier":
						_read_tier_1(igt, e);
						break;

					default:
						throw not.expected;
				}
			}

			foreach (XmlElement e in e_igt.ChildNodes)
			{
				switch (e.Name)
				{
					case "metadata":
						break;

					case "tier":
						_read_tier_2(igt, e);
						break;

					default:
						throw not.expected;
				}
			}

			//igt.CoerceValue(igt.FromLineProperty);
			//igt.CoerceValue(igt.ToLineProperty);
			Nop.X();
		}

		void _read_tier_1(igt igt, XmlElement e_tier)
		{
			var tier = new content_tier
			{
				id = e_tier.GetAttribute("id"),
				type = e_tier.GetAttribute("type"),
				state = e_tier.GetAttribute("state"),
			};
			igt.Add(tier);

			foreach (XmlElement e in e_tier.ChildNodes)
			{
				switch (e.Name)
				{
					case "metadata":
						_read_metadata(tier, e);
						break;

					case "item":
						_read_item_1(tier, e);
						break;

					default:
						throw not.expected;
				}
			}
		}

		void _read_item_1(tier t, XmlElement e_item)
		{
			String s_attr;

			var ii = new item
			{
				id = e_item.GetAttribute("id"),
				type = e_item.GetAttribute("type"),
			};

			if ((s_attr = e_item.InnerText).Length > 0)
			{
				ii.content = new tref_content { Text = s_attr };
			}

			t.Add(ii);

#if false
			if (t == null)
			{
				if (_it.HasAttribute("line"))
					t = new TextGroupTier();
				else if (_it.HasAttribute("content"))
					t = new SegTier();
				else
					throw not.expected;

				id_map.Add(t.Name = e_item.GetAttribute("id"), t);
				t.TierType = e_item.GetAttribute("type");
			}
#endif
			//s_attr = e_item.GetAttribute("id");

#if false
			if (t is TextGroupTier)
			{
				var tt = new TextTier
				{
					Text = _it.InnerText,
					LineNumbers = _it.GetAttribute("line")
									.Split(alib.Character.Charset.ws, StringSplitOptions.RemoveEmptyEntries)
									.Select(x => int.Parse(x))
									.ToArray(),
					TierType = s_attr,
				};
				((TextGroupTier)t).Add(tt);
				hi = tt;
			}
			else if (t is SegTier)
			{
				var z = _it.GetAttribute("content");

				//Debug.WriteLine(z);

				var rgs = z.Split(rgch_standoff_split, StringSplitOptions.RemoveEmptyEntries);
				if (rgs.Length != 3)
					throw not.expected;

				var sp = new SegPart
				{
					//Text = _it.InnerText
					//Name = _it.GetAttribute("id"),
					SourceTier = (TextTier)id_map[rgs[0]],
					FromChar = int.Parse(rgs[1]),
					ToChar = int.Parse(rgs[2]),
				};
				((SegTier)t).Add(sp);
				hi = sp;
			}
			else
			{
				throw not.expected;
			}
#endif

			//base.Add(ii.id = s_attr, ii);
		}

		void _read_tier_2(igt igt, XmlElement e_tier)
		{
			content_tier tier;
			String s_attr;
			var map = igt.name_map;

			var t = map[e_tier.GetAttribute("id")];

			if ((tier = t as content_tier) != null)
			{
				if ((s_attr = e_tier.GetAttribute("segmentation")).Length > 0)
				{
					if (e_tier.HasAttribute("alignment") || e_tier.HasAttribute("content"))
						throw not.valid;

					tier.content = tier.alignment = (tier)map[s_attr];
				}
				else
				{
					if ((s_attr = e_tier.GetAttribute("content")).Length > 0)
						tier.content = (tier)map[s_attr];

					if ((s_attr = e_tier.GetAttribute("alignment")).Length > 0)
						tier.alignment = (tier)map[s_attr];
				}
			}
			else
			{
				throw not.impl;
			}

			foreach (XmlElement e in e_tier.ChildNodes)
			{
				switch (e.Name)
				{
					case "item":
						_read_item_2(tier, e);
						break;

					default:
						throw not.expected;
				}
			}
		}

		void _read_item_2(tier tier, XmlElement e_item)
		{
			String s_attr;

			var item = (item)tier.Host.name_map[e_item.GetAttribute("id")];

			if ((s_attr = e_item.GetAttribute("segmentation")).Length > 0)
			{
				if (item.content != null || e_item.HasAttribute("content") || e_item.HasAttribute("alignment"))
					throw not.valid;

				item.alignment = item.content = tier.Host.ParseRef(s_attr);
			}
			else
			{
				if ((s_attr = e_item.GetAttribute("content")).Length > 0)
				{
					if (item.content != null)
						throw not.expected;

					item.content = tier.Host.ParseRef(s_attr);
				}

				if ((s_attr = e_item.GetAttribute("alignment")).Length > 0)
				{
					item.alignment = tier.Host.ParseRef(s_attr);
				}
			}
		}
	};
}
