﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Markup;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.Reflection;

namespace xigt
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	[DebuggerDisplay("{ToString(),nq}")]
	public class _elem : XmlElement, IElem
	{
		public _elem(String prefix, String name, String s_ns, corpus_xmldoc xd)
			: base(prefix, name, s_ns, xd)
		{
		}

		new public corpus_xmldoc OwnerDocument { get { return (corpus_xmldoc)base.OwnerDocument; } }

		public virtual void OnXmlLoaded()
		{
			OnPropertyChanged("id");
			OnPropertyChanged("type");
		}

		public virtual String id
		{
			get { return GetAttribute("id").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.id)
				{
					SetAttribute("id", value);
					OnPropertyChanged("id");
				}
			}
		}

		public String type
		{
			get { return GetAttribute("type").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.type)
				{
					SetAttribute("type", value);
					OnPropertyChanged("type");
				}
			}
		}

		IHost IElem.Host { get { return (IHost)base.ParentNode; } }

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(String s)
		{
			var _tmp = PropertyChanged;
			if (_tmp != null)
				_tmp(this, new PropertyChangedEventArgs(s));
		}

		public override String ToString()
		{
			return String.Format("{0} {1} {2}",
				GetType().Name,
				id ?? "(null-id)",
				type);
		}
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public abstract class _elem<H> : _elem, IElem<H>
		where H : class, IHost<IElem<H>>
	{
		public _elem(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
		}

		public H Host { [DebuggerStepThrough] get { return base.ParentNode as H; } }
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public abstract class _elem_host<H, T> : _elem<H>, IHost
		where H : class, IHost<IElem<H>>
		where T : _elem, IElem<IHost<T>>
	{
		public _elem_host(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
		}

		public void RemoveAt(int ix) { throw not.impl; }

		Object IList.this[int ix]
		{
			get { return ((IList<T>)this)[ix]; }
			set { ((IList<T>)this)[ix] = (T)value; }
		}

		int IList.Add(Object value)
		{
			((IList<T>)this).Add((T)value);
			return ((IList<T>)this).Count - 1;
		}

		void IList.Clear() { ((IList<T>)this).Clear(); }

		int ICollection.Count { get { return ((IList<T>)this).Count; } }

		void IList.Insert(int ix, Object item) { ((IList<T>)this).Insert(ix, (T)item); }

		void IList.Remove(Object item) { ((IList<T>)this).Remove((T)item); }

		bool IList.Contains(Object item) { return ((IList<T>)this).Contains((T)item); }

		int IList.IndexOf(Object item) { return ((IList<T>)this).IndexOf((T)item); }

		IEnumerator IEnumerable.GetEnumerator() { return ((IList<T>)this).GetEnumerator(); }

		void ICollection.CopyTo(Array array, int ix)
		{
			var e = ((IList<T>)this).GetEnumerator();
			while (e.MoveNext())
				array.SetValue(e.Current, ix++);
		}

		bool IList.IsFixedSize { get { return false; } }
		bool ICollection.IsSynchronized { get { return false; } }
		Object ICollection.SyncRoot { get { return null; } }

		public event NotifyCollectionChangedEventHandler CollectionChanged;

		public virtual void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
		{
			var _tmp = CollectionChanged;
			if (_tmp != null)
				_tmp(this, e);
		}

		public virtual void ReplaceChild(T _new, T _old)
		{
			base.ReplaceChild(_new, _old);

			if (_new.Host != this || _old.Host != null)
				throw not.expected;
		}
	};


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public abstract class _host<H, T> : _elem_host<H, T>, IList<T>, IElemHost<H, T>
		where H : class, IHost<IElem<H>>
		where T : _elem, IElem<IHost<T>>
	{
		public _host(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
		}

		T selected_item;
		public T SelectedItem
		{
			get { return selected_item; }
			set
			{
				if (value != selected_item)
				{
					selected_item = value;
					OnPropertyChanged("SelectedItem");
				}
			}
		}

		public override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
		{
			base.OnCollectionChanged(e);

			var n = FirstOrDefault();
			if (n == null)
				SelectedItem = null;
			else if (SelectedItem == null)
				SelectedItem = n;
		}

		public override void OnXmlLoaded()
		{
			base.OnXmlLoaded();

			foreach (var t in this)
				t.OnXmlLoaded();

			OnPropertyChanged("Item[]");
			OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
		}

		public bool ContainsId(String s_id)
		{
			foreach (var t in this)
				if (t.id == s_id)
					return true;
			return false;
		}

		public int Count
		{
			get
			{
				int c = 0;
				for (var n = FirstChild; n != null; n = n.NextSibling)
					if (n is T)
						c++;
				return c;
			}
		}

		public void Add(T item) { Insert(-1, item); }
		public void Insert(int ix, T item)
		{
			int c;

			if (FirstChild == null)
				ix = c = 0;
			else
			{
				c = Count;
				if (ix == -1)
					ix = c;
			}

			if (ix == c)
				this.AppendChild(item);
			else
				this.InsertBefore(item, this[ix]);

			base.OnPropertyChanged("Count");
			base.OnPropertyChanged("Item[]");
			OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, item, ix));
		}

		public bool Remove(T item)
		{
			var ix = IndexOf(item);
			if (ix == -1)
				return false;

			this.RemoveChild(item);

			base.OnPropertyChanged("Count");
			base.OnPropertyChanged("Item[]");
			OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Remove, item, ix));
			return true;
		}

		public void Move(int oldIndex, int newIndex)
		{
			throw not.impl;
		}

		public void Clear()
		{
			XmlNode _next, n = FirstChild;
			bool f_any = false;
			while (n != null)
			{
				_next = n.NextSibling;
				if (n is T)
				{
					base.RemoveChild(n);
					f_any = true;
				}
				n = _next;
			}
			if (f_any)
			{
				base.OnPropertyChanged("Count");
				base.OnPropertyChanged("Item[]");
				OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));
			}
		}

		public T this[int ix]
		{
			get
			{
				T t;
				for (var n = FirstChild; n != null; n = n.NextSibling)
					if ((t = n as T) != null && --ix < 0)
						return t;
				throw new IndexOutOfRangeException();
			}
			set
			{
				throw not.impl;
			}
		}

		public bool Contains(T item) { return IndexOf(item) != -1; }

		public int IndexOf(T item)
		{
			if (item.ParentNode == this)
			{
				int ix = 0;
				for (var n = FirstChild; n != null; n = n.NextSibling)
					if (n is T)
					{
						if (n == item)
							return ix;
						ix++;
					}
			}
			return -1;
		}

		public void CopyTo(T[] array, int ix)
		{
			for (var n = FirstChild; n != null; n = n.NextSibling)
				if ((array[ix] = n as T) != null)
					ix++;
		}

		new public IEnumerator<T> GetEnumerator()
		{
			T t;
			for (var n = FirstChild; n != null; n = n.NextSibling)
				if ((t = n as T) != null)
					yield return t;
		}

		public T FirstOrDefault()
		{
			T t;
			for (var n = FirstChild; n != null; n = n.NextSibling)
				if ((t = n as T) != null)
					return t;
			return null;
		}


		public override void ReplaceChild(T _new, T _old)
		{
			var ix = IndexOf(_old);
			if (ix == -1)
				throw not.valid;

			base.ReplaceChild(_new, _old);

			Debug.Print("not good enough: need to update ID databases");
			_new.OnXmlLoaded();

			OnPropertyChanged("Item[]");
			OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Replace, _new, _old, ix));

			Debug.Print("replaced {0} at index {1} in {2} id={3}, new id={4}", _new.GetType().Name, ix, this.GetType().Name, this.id, _new.id);
		}
	};
}
