﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Text;
using System.Windows.Markup;
using System.Xaml;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Enumerable;

namespace xigt
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class corpora : ObservableCollection<xigt_corpus>, IHost<xigt_corpus>
	{
		public xigt_corpus FindCorpus(String fn)
		{
			fn = new Uri(Path.GetFullPath(fn)).LocalPath;

			foreach (var xc in this)
				if (String.Equals(xc.FullPath, fn, StringComparison.Ordinal))
					return xc;
			return null;
		}

		public AddCorpusResult AddXigtFile(String fn)
		{
			xigt_corpus _;
			return AddXigtFile(fn, out _);
		}
		public AddCorpusResult AddXigtFile(String fn, out xigt_corpus xc)
		{
			xc = null;
			fn = new Uri(Path.GetFullPath(fn)).LocalPath;

			if (!File.Exists(fn))
				return AddCorpusResult.FileNotFound;

			if ((xc = FindCorpus(fn)) != null)
				return AddCorpusResult.AlreadyLoaded;

			xc = xigt_corpus.Load(fn);
			base.Add(xc);

			return AddCorpusResult.Ok;
		}

		protected override void InsertItem(int index, xigt_corpus item)
		{
			if (item.FullPath == null)
				item.CorpusName = next_corpus_name();

			base.InsertItem(index, item);
		}

		protected override void RemoveItem(int index)
		{
			var xc = base[index];
			if (xc == selected_item)
			{
				if (Count == 1)
					SelectedItem = null;
				else if (index == Count - 1)
					SelectedItem = base[index - 1];
				else
					SelectedItem = base[index + 1];
			}

			base.RemoveItem(index);

			xc.FullPath = null;
		}

		xigt_corpus selected_item;
		public xigt_corpus SelectedItem
		{
			get { return selected_item; }
			set
			{
				if (value != selected_item)
				{
					selected_item = value;
					base.OnPropertyChanged(new PropertyChangedEventArgs("SelectedItem"));
				}
			}
		}

		String next_corpus_name()
		{
			String s_name;
			int i = 0;
			do
				s_name = "corpus" + ++i + ".xml";
			while (this.Any(x => String.Equals(x.CorpusName, s_name, StringComparison.OrdinalIgnoreCase)));
			return s_name;
		}

		protected override void OnCollectionChanged(NotifyCollectionChangedEventArgs e)
		{
			base.OnCollectionChanged(e);

			if (Count == 0)
				SelectedItem = null;
			else if (SelectedItem == null)
				SelectedItem = base[0];
		}

		public String id
		{
			get { throw not.impl; }
			set { throw not.impl; }
		}
		public String type
		{
			get { throw not.impl; }
			set { throw not.impl; }
		}
	};


	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class TestCorpora : corpora
	{
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		static TestCorpora _inst;

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static TestCorpora corpora
		{
			[DebuggerStepThrough]
			get
			{
				if (_inst == null)
				{
					_inst = new TestCorpora();
					_inst.AddXigtFile(@"d:\programming\xigt-edit\!test-xigt\kor-ex.xml");
				}
				return _inst;
			}
		}

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static xigt_corpus xigt_corpus { [DebuggerStepThrough] get { return corpora.SelectedItem; } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static igt igt { [DebuggerStepThrough] get { return xigt_corpus.SelectedItem; } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static tier tier { [DebuggerStepThrough] get { return igt.SelectedItem; } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static tier ps_tier { [DebuggerStepThrough] get { return igt.First(x => x.Mode == tier_mode.PhraseStructure); } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static tier dep_tier { [DebuggerStepThrough] get { return igt.First(x => x.Mode == tier_mode.Dependency); } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static tier bitext_tier { [DebuggerStepThrough] get { return igt.First(x => x.Mode == tier_mode.BilingualAlignment); } }

		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public static item item { [DebuggerStepThrough] get { return tier.SelectedItem; } }
	};
}