﻿using System;
using System.Diagnostics;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Markup;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.String;

namespace xigt
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class RefMap : Dictionary<String, IElem>		//XmlNamedNodeMap ??
	{
		public RefMap(igt igt)
		{
			this.igt = igt;

			foreach (var tier in igt)
			{
				base.Add(tier.id, tier);

				foreach (var item in tier)
				{
					base.Add(item.id, item);
				}
			}
		}

		readonly public igt igt;

		public String ResolveRef(String s_ref)
		{
			var inner = s_ref.ExtractFirstBracket();

			var id = s_ref.RemoveBracketed();

			var id_txt = ((item)base[id]).Text;

			String[] rg;
			if (inner != null && (rg = inner.Split(':')).Length == 2)
			{
				int i_from = int.Parse(rg[0]);
				int i_to = int.Parse(rg[1]);
				id_txt = id_txt.Substring(i_from, i_to - i_from);
			}

			return id_txt;
		}

		public String GetTierId()
		{
			String s_id;
			for (int n = 1; ; n++)
			{
				for (Char ch = 'z'; ch >= 'a'; ch--)
				{
					if (!base.ContainsKey(s_id = new String(ch, n)))
						return s_id;
				}
			}
		}

		public String GetItemId(tier t)
		{
			String s_id;
			for (int n = 1; ; n++)
			{
				if (!base.ContainsKey(s_id = t.id + n))
					return s_id;
			}
		}
	};

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class igt : _host<xigt_corpus, tier>
	{
		public igt(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
		}

		public RefMap igt_names;

		[DefaultValue(default(String))]
		public String DocId
		{
			get { return GetAttribute("doc-id"); }
			set
			{
				value = value.Trim();
				if (value != GetAttribute("doc-id"))
				{
					SetAttribute("doc-id", value);
					OnPropertyChanged("DocId");
				}
			}
		}

		[DefaultValue(default(String))]
		public String DocInfo
		{
			get { return ""; }
			set { }
		}

		[DefaultValue(default(String))]
		public String Language
		{
			[DebuggerStepThrough]
			get { return ""; }
			set { }
		}

		public override String id
		{
			get { return base.id; }
			set
			{
				String cur;

				value = value.Trim();
				if (value != (cur = base.id))
				{
					if (value.Length > 0 && Host.ContainsId(value))
						throw new DuplicateKeyException();

					base.id = value;
				}
			}
		}

		public tier InsertTier(int ix)
		{

			var e_tier = (tier)OwnerDocument.CreateElement("tier");
			e_tier.id = igt_names.GetTierId();

			var e_item = (item)OwnerDocument.CreateElement("item");
			e_item.id = igt_names.GetItemId(e_tier);
			e_item.InnerText = "new item text";
			e_tier.Add(e_item);

			this.Insert(ix, e_tier);

			return e_tier;
		}

		public override void OnXmlLoaded()
		{
			this.igt_names = new RefMap(this);

			base.OnXmlLoaded();
		}

#if TREF
		static char[] spl1 = { ']', ' ', '\t', '\r', '\n' };

		public tref ParseRef(String s)
		{
			int i, i_last = 0;
			bool f_in = false;

			tref e = null;
			item link = null;
			int i0 = 0, i1 = int.MaxValue;
			//String delim = null;
			for (i = 0; i < s.Length; i++)
			{
				switch (s[i])
				{
					case '[':
						if (f_in || i0 != 0 || i1 != int.MaxValue)
							throw not.valid;
						f_in = true;
						link = (item)igt_names[s.Substring(i_last, i - i_last)];
						i_last = i + 1;
						break;

					case ':':
						if (!f_in || i0 != 0 || i1 != int.MaxValue)
							throw not.valid;
						i0 = int.Parse(s.Substring(i_last, i - i_last));
						i_last = i + 1;
						break;

					case ']':
						if (!f_in || i1 != int.MaxValue)
							throw not.valid;
						f_in = false;
						i1 = int.Parse(s.Substring(i_last, i - i_last));
						i_last = i + 1;

						e = new tref(link, i0, i1);

						i0 = 0;
						i1 = int.MaxValue;
						link = null;
						break;

					case '+':
						throw not.impl;

					case ',':
						throw not.impl;

					default:
						break;
				}
			}
			if (f_in)
				throw not.valid;

			if (i == 0)
				return null;

			if ((i > i_last) != (e == null))
				throw not.valid;

			if (i > i_last)
			{
				if (i0 != 0 || i1 != int.MaxValue)
					throw not.expected;

				link = (item)igt_names[s.Substring(i_last, i - i_last)];

				e = new tref(link);

				//e.CoerceValue(tref.TextProperty);

				//item = (item)name_map[s.Substring(i_last, i - i_last)];
			}

			return e;
			//if ((cb = i - i_last) > 0)
			//	yield return s.Substring(i_last, cb);
		}
#endif
	};
}
