﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Data;
using System.Windows.Markup;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Array;
using alib.Enumerable;
using alib.Bits;
using alib.Collections;
using alib.Character;

namespace xigt
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class item : _elem<tier>
	{
		public item(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
			this.Mode = item_mode.Unknown;
		}

		public override void OnXmlLoaded()
		{
			base.OnXmlLoaded();
		}

		void determine_mode()
		{
			if (HasAttribute("dep") || HasAttribute("head"))
			{
				_mode = item_mode.Dependency;
			}
			else if (HasAttribute("source") || HasAttribute("target"))
			{
				_mode = item_mode.BilingualAlignment;
			}
			else if (HasAttribute("children"))
			{
				_mode = item_mode.PSNode;
			}
			else if (HasAttribute("segmentation"))
			{
				_mode = item_mode.Segmentation;
			}
			else if (HasAttribute("content"))
			{
				if (HasAttribute("alignment"))
					_mode = item_mode.Annotation;
				else
					_mode = item_mode.Alias;
			}
			else if (HasAttribute("alignment"))
			{
				_mode = item_mode.Align;
			}
			else
			{
				_mode = item_mode.Source;
			}
		}

		item_mode _mode;
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public item_mode Mode
		{
			get
			{
				if (_mode == item_mode.Unknown)
					determine_mode();
				return _mode;
			}
			set
			{
				if (_mode != value)
				{
					_mode = value;
					OnPropertyChanged("Mode");
				}
			}
		}

		public String Text
		{
			get
			{
				String s_ref;
				if ((s_ref = GetAttribute("content")).Length > 0 || (s_ref = GetAttribute("segmentation")).Length > 0)
				{
					//return "@" + s_txt;
					return this.GetIgt().igt_names.ResolveRef(s_ref);
				}
				return InnerText;
			}
			set
			{
				if (HasAttribute("content"))
					throw not.valid;
				if (value != InnerText)
				{
					InnerText = value;
					OnPropertyChanged("Text");
				}
			}
		}

		public bool CanEdit
		{
			get
			{
				return GetAttribute("content").Length + GetAttribute("segmentation").Length == 0;
			}
		}

		public String content
		{
			get
			{
				if (Mode == item_mode.Segmentation)
					return GetAttribute("segmentation").Trim();
				else
					return GetAttribute("content").Trim();
			}
			set
			{
				value = value.Trim();
				if (value != this.content)
				{
					if (Mode == item_mode.Segmentation)
						SetAttribute("segmentation", value);
					else
						SetAttribute("content", value);
					OnPropertyChanged("content");
					OnPropertyChanged("Text");
				}
			}
		}

		public String alignment
		{
			get { return GetAttribute("alignment").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.alignment)
				{
					SetAttribute("alignment", value);
					OnPropertyChanged("alignment");
					OnPropertyChanged("AlignmentText");
				}
			}
		}

		public String AlignmentText
		{
			get
			{
				String s_ref;
				if ((s_ref = GetAttribute("alignment")).Length == 0)
					return "";
				return this.GetIgt().igt_names.ResolveRef(s_ref);
			}
		}

		public String dep
		{
			get { return GetAttribute("dep").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.dep)
				{
					SetAttribute("dep", value);
					OnPropertyChanged("dep");
					OnPropertyChanged("DependencyText");
				}
			}
		}

		public String DependencyText
		{
			get
			{
				String s_ref;
				if ((s_ref = GetAttribute("dep")).Length == 0)
					return "";
				return this.GetIgt().igt_names.ResolveRef(s_ref);
			}
		}

		public String head
		{
			get { return GetAttribute("head").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.head)
				{
					SetAttribute("head", value);
					OnPropertyChanged("head");
				}
			}
		}

		public item parent
		{
			get
			{
				String s_children;
				String[] rgs;
				var this_id = this.id;
				if (this_id.Length > 0)
				{
					foreach (var it in Host)
					{
						if (it == this)
							continue;
						if ((s_children = it.GetAttribute("children")).Length > 0)
						{
							rgs = s_children.Split(Charset.comma_semi_ws, StringSplitOptions.RemoveEmptyEntries);

							if (rgs.Contains(this_id, StringComparer.OrdinalIgnoreCase))
								return it;
						}
					}
				}
				return null;
			}
			//set
			//{
			//	_align = value;
			//}
		}

		public String source
		{
			get { return GetAttribute("source").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.source)
				{
					SetAttribute("source", value);
					OnPropertyChanged("source");
					OnPropertyChanged("SourceText");
				}
			}
		}

		public String SourceText
		{
			get
			{
				String s_ref;
				if ((s_ref = GetAttribute("source")).Length == 0)
					return "";
				return this.GetIgt().igt_names.ResolveRef(s_ref);
			}
		}

		public String target
		{
			get { return GetAttribute("target").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.target)
				{
					SetAttribute("target", value);
					OnPropertyChanged("target");
					OnPropertyChanged("TargetText");
				}
			}
		}

		//public String TargetText
		//{
		//	get
		//	{
		//		String s_ref;
		//		if ((s_ref = GetAttribute("target")).Length == 0)
		//			return "";
		//		return this.GetIgt().igt_names.ResolveRef(s_ref);
		//	}
		//}

		public String line
		{
			get { return GetAttribute("line").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.line)
				{
					SetAttribute("line", value);
					OnPropertyChanged("line");
				}
			}
		}

		public String tag
		{
			get { return GetAttribute("tag").Trim(); }
			set
			{
				value = value.Trim();
				if (value != this.tag)
				{
					SetAttribute("tag", value);
					OnPropertyChanged("tag");
				}
			}
		}

		public override String ToString()
		{
			var s = base.ToString();
			s += " " + this.Text;
			return s;
		}
	};
}
