﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Markup;
using System.Xml;

using alib;
using alib.Debugging;
using alib.Enumerable;
using alib.String;

namespace xigt
{
	///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/// 
	public class tier : _host<igt, item>
	{
		public tier(String pfx, String name, String s_ns, corpus_xmldoc xd)
			: base(pfx, name, s_ns, xd)
		{
			this._mode = tier_mode.Unknown;
			this._orientation = TierOrientation.Default;
		}

		public override void OnXmlLoaded()
		{
			base.OnXmlLoaded();
			OnPropertyChanged("state");

			String s_attr;
			var map = Host.igt_names;

			if ((s_attr = GetAttribute("segmentation")).Length > 0)
			{
				if (HasAttribute("alignment") || HasAttribute("content"))
					throw not.valid;

				this.content = this.alignment = (tier)map[s_attr];
			}
			else
			{
				if ((s_attr = GetAttribute("content")).Length > 0)
					this.content = (tier)map[s_attr];

				if ((s_attr = GetAttribute("alignment")).Length > 0)
					this.alignment = (tier)map[s_attr];
			}

			//	if (item.line.Length > 0)
			//		_item_line_nums = true;
			//	if (item.tag.Length > 0)
			//		_item_tags = true;
			//}
			//SetValue(ShowAlignRefPropertyKey, t != null && t.alignment != null && t.alignment != t.content);
			//SetValue(ShowContentRefPropertyKey, t != null && t.content != null);

			_item_line_nums = _item_tags = false;

			foreach (var item in this)
			{
				if (item.line.Length > 0)
					_item_line_nums = true;
				if (item.tag.Length > 0)
					_item_tags = true;
			}
		}

		[DefaultValue(default(String))]
		public String state
		{
			get { return GetAttribute("state"); }
			set
			{
				value = value.Trim();
				if (value != GetAttribute("state"))
				{
					SetAttribute("state", value);
					OnPropertyChanged("state");
				}
			}
		}

		void determine_mode()
		{
			if (this.Any(x => x.Mode == item_mode.Dependency))
			{
				_mode = tier_mode.Dependency;
			}
			else if (this.Any(x => x.Mode == item_mode.BilingualAlignment))
			{
				_mode = tier_mode.BilingualAlignment;
			}
			else if (this.Any(x => x.Mode == item_mode.PSNode))
			{
				_mode = tier_mode.PhraseStructure;
			}
			else if (this.Any(x => x.Mode == item_mode.Segmentation))
			{
				_mode = tier_mode.Segmentation;
			}
			else if (this.Any(x => x.Mode == item_mode.Alias))
			{
				_mode = tier_mode.Alias;
			}
			else if (this.Any(x => x.Mode == item_mode.Annotation))
			{
				_mode = tier_mode.Annotation;
			}
			else if (this.Any(x => x.Mode == item_mode.Align))
			{
				_mode = tier_mode.Align;
			}
			else
			{
				_mode = tier_mode.Source;
			}
		}

		tier_mode _mode;
		[DebuggerBrowsable(DebuggerBrowsableState.Never)]
		public tier_mode Mode
		{
			get
			{
				if (_mode == tier_mode.Unknown)
					determine_mode();
				return _mode;
			}
			set
			{
				if (_mode != value)
				{
					_mode = value;
					OnPropertyChanged("Mode");
				}
			}
		}

		public tier content
		{
			get
			{
				String s;
				if ((s = GetAttribute("content")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.content)
				{
					SetAttribute("content", value.id);
					OnPropertyChanged("content");
				}
			}
		}

		public tier alignment
		{
			get
			{
				String s;
				if ((s = GetAttribute("alignment")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.alignment)
				{
					SetAttribute("alignment", value.id);
					OnPropertyChanged("alignment");
				}
			}
		}

		public tier dep
		{
			get
			{
				String s;
				if ((s = GetAttribute("dep")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.dep)
				{
					SetAttribute("dep", value.id);
					OnPropertyChanged("dep");
				}
			}
		}

		public tier head
		{
			get
			{
				String s;
				if ((s = GetAttribute("head")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.head)
				{
					SetAttribute("head", value.id);
					OnPropertyChanged("head");
				}
			}
		}

		public tier source
		{
			get
			{
				String s;
				if ((s = GetAttribute("source")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.source)
				{
					SetAttribute("source", value.id);
					OnPropertyChanged("source");
				}
			}
		}

		public tier target
		{
			get
			{
				String s;
				if ((s = GetAttribute("target")).Length > 0)
				{
					IElem el;
					igt igt;
					if ((igt = this.GetIgt()) != null && igt.igt_names.TryGetValue(s, out el))
						return el as tier;
				}
				return null;
			}
			set
			{
				if (value != this.target)
				{
					SetAttribute("target", value.id);
					OnPropertyChanged("target");
				}
			}
		}

		public String FormattedXml
		{
			get
			{
				var sb = new StringBuilder();
				using (var xw = XmlWriter.Create(sb, new XmlWriterSettings
				{
					Indent = true,
					NewLineOnAttributes = false,
					OmitXmlDeclaration = true,
				}))
				{
					this.WriteTo(xw);
				}
				return sb.ToString();
			}
		}

		public bool TrySetRawXml(String s_xml)
		{
			var xdf = OwnerDocument.CreateDocumentFragment();

			xdf.InnerXml = s_xml;

			var _new = (tier)xdf.FirstChild;
			var _host = this.Host;

			_host.ReplaceChild(_new, this);

			return true;
		}

		protected override void OnPropertyChanged(String s)
		{
			base.OnPropertyChanged(s);
			if (s == "type" && _orientation == TierOrientation.Default)
				base.OnPropertyChanged("TierOrientation");
		}

		TierOrientation _orientation;
		public TierOrientation TierOrientation
		{
			get
			{
				if (_orientation == TierOrientation.Default)
					return type.StartsWith("odin") ? TierOrientation.Vertical : TierOrientation.Horizontal;
				return _orientation;
			}
			set
			{
				if (value != _orientation)
				{
					_orientation = value;
					base.OnPropertyChanged("TierOrientation");
				}
			}
		}

		//bool _item_content_refs;
		//public bool AnyItemContentRefs
		//{
		//	get { return _item_content_refs; }
		//	set
		//	{
		//		if (value != _item_content_refs)
		//		{
		//			_item_content_refs = value;
		//			base.OnPropertyChanged("AnyItemContentRefs");
		//		}
		//	}
		//}

		//bool _item_align_refs;
		//public bool AnyItemAlignRefs
		//{
		//	get { return _item_align_refs; }
		//	set
		//	{
		//		if (value != _item_align_refs)
		//		{
		//			_item_align_refs = value;
		//			base.OnPropertyChanged("AnyItemAlignRefs");
		//		}
		//	}
		//}

		bool _item_line_nums;
		public bool AnyItemLineNumbers
		{
			get { return _item_line_nums; }
			set
			{
				if (value != _item_line_nums)
				{
					_item_line_nums = value;
					base.OnPropertyChanged("AnyItemLineNumbers");
				}
			}
		}

		bool _item_tags;
		public bool AnyItemTags
		{
			get { return _item_tags; }
			set
			{
				if (value != _item_tags)
				{
					_item_tags = value;
					base.OnPropertyChanged("AnyItemTags");
				}
			}
		}
	};
}
